unit uExplorer;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ImgList, dOPC, Menus;
const
  WM_ADDITEMS = WM_USER + 1;

type
  TExplorerDlg = class(TForm)
    TopPanel: TPanel;
    TreeView1: TTreeView;
    BottomPanel: TPanel;
    Splitter1: TSplitter;
    ImageList1: TImageList;
    Label2: TLabel;
    Label3: TLabel;
    ListBox1: TListBox;
    PopupMenu1: TPopupMenu;
    ItemstoInspector1: TMenuItem;
    procedure TreeView1Change(Sender: TObject; Node: TTreeNode);
    procedure FormShow(Sender: TObject);
    procedure ItemstoInspector1Click(Sender: TObject);
    procedure ListBox1DblClick(Sender: TObject);
  private
    FOPCServer: TdOPCServer;
    procedure ShowLeafsInListBox;
  public
    procedure ShowOPCPath;
    procedure HideOPCPath;
    procedure GetSelectedItemIds(ItemIds: TStrings);
    property  OPCServer: TdOPCServer read FOPCServer write FOPCServer ;
  end;

var
  ExplorerDlg: TExplorerDlg;

implementation

{$R *.DFM}

// fills a Treeview with all Branches of the OPC Server
//@params Browser  a created TdvOPCBrowser object
//@params Tree     Treeview to fill
//@params Node     call always with nil
procedure BranchesToTreeView(Browser: TdOPCBrowser; Tree: TTreeView; Node : TTreeNode = nil);
var
  i       : integer;
  S       : TStringList;
  NewNode : TTreeNode;
begin
  Browser.ShowBranches;            // get all Branches in this level from OPC Server
  S := TStringList.Create;
  S.Assign(Browser.Items);         // save Items in new StringList
  for i := 0 to S.Count-1 do       // for all Items in stringlist
  begin
      NewNode := Tree.Items.AddChild(Node,S[i]);
      if assigned(Tree.Images) then                // only for selected images
        if Tree.Images.Count > 1 then
          NewNode.SelectedIndex := 1;
      if Browser.MoveDown(S[i]) then               // one Level down
      begin
        BranchesToTreeView(Browser,Tree,NewNode);  // rekusive call
        Browser.MoveUp;                            // back to old Level
      end;
  end;
  S.Free;
end;

// get complete Path delimited with <Delimiter> for the selected Node
// Test.Test.Test
function GetTreeViewPathStr(Tree: TTreeview; Delimiter: char): string;
var
  node    : TTreeNode;
  PathStr : string;
  D       : string;
begin
  D := Delimiter;
  Node := Tree.Selected;
  PathStr := '';
  if Node <> nil then
  begin
    repeat
      PathStr :=  '"'+Node.Text+'"'+ D + PathStr ;
      Node := Node.Parent;
    until Node = nil;
    delete(PathStr,length(PathStr),1);  // delete last delimiter
  end;
  result := PathStr;
end;

procedure TExplorerDlg.ShowLeafsInListBox;
var
  PathList  : TStrings;
  i         : integer;
begin
  PathList := TStringlist.Create;                          // create new Stringlist
  PathList.Commatext := GetTreeViewPathStr(TreeView1,','); //
  OpcServer.Browser.MovetoRoot;                            // Move to Top
  for i := 0 to PathList.Count-1 do                        // now Move down step by step
    OpcServer.Browser.MoveDown(PathList[i]);
  OpcServer.Browser.ShowLeafs;                             // get all Items in Path
  ListBox1.Items.Assign(OpcServer.Browser.Items);          // Show in Listbox
  PathList.Free;
end;

procedure TExplorerDlg.TreeView1Change(Sender: TObject; Node: TTreeNode);
begin
  if OPCServer.Active then    // only if connected to opc server
    ShowLeafsInListBox;    // if Tree change then show leafs in Path
end;

procedure TExplorerDlg.FormShow(Sender: TObject);
begin
 HideOPCPath;
end;

procedure TExplorerDlg.ShowOPCPath;
begin
  BranchesToTreeView(OpcServer.Browser,TreeView1); // get all Pathes from OPC Server
  ShowLeafsInListBox;                             // Show Items in Path
end;

procedure TExplorerDlg.HideOPCPath;
begin
  TreeView1.Items.Clear; // clear all lists
  ListBox1.Items.Clear;
end;

procedure TExplorerDlg.GetSelectedItemIds(ItemIds: TStrings);
var
  i : integer;
begin
 ItemIds.Clear;
 for i := 0 to Listbox1.Items.Count -1 do
   if Listbox1.Selected[i] then
     ItemIds.Add(OPCServer.Browser.GetItemId(ListBox1.Items[i]));
end;

// inform parentwindow that user want to add items to inspector
procedure TExplorerDlg.ItemstoInspector1Click(Sender: TObject);
var
  SelCount,i : integer;
begin
   SelCount := 0;
   for i := 0 to Listbox1.Items.Count -1 do
     if Listbox1.Selected[i] then
       Inc(SelCount);
   if SelCount > 0 then
     PostMessage(Application.MainForm.Handle,WM_ADDITEMS,0,0);
end;

procedure TExplorerDlg.ListBox1DblClick(Sender: TObject);
begin
 ItemstoInspector1Click(Sender);
end;

end.
