unit uMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls, ComCtrls, ImgList, ScktComp, dOPC;

type
  TForm1 = class(TForm)
    TreeView1: TTreeView;
    Panel2: TPanel;
    Splitter1: TSplitter;
    TopPanel: TPanel;
    Label1: TLabel;
    Label2: TLabel;
    ServerCombo: TComboBox;
    bConnect: TButton;
    bDisconnect: TButton;
    Label3: TLabel;
    eGroupName: TEdit;
    Label4: TLabel;
    cActive: TCheckBox;
    eUpdateRate: TEdit;
    Label5: TLabel;
    eDeadBand: TEdit;
    Label6: TLabel;
    bAdd: TButton;
    bChange: TButton;
    bDelete: TButton;
    Timer1: TTimer;
    Panel1: TPanel;
    Label7: TLabel;
    Label8: TLabel;
    LName: TLabel;
    Label9: TLabel;
    LUpdate: TLabel;
    Label10: TLabel;
    LDeadBand: TLabel;
    Label11: TLabel;
    LActive: TLabel;
    Panel3: TPanel;
    Label12: TLabel;
    Label14: TLabel;
    LServer: TLabel;
    Label16: TLabel;
    LStartTime: TLabel;
    Label17: TLabel;
    LState: TLabel;
    Label18: TLabel;
    LInfo: TLabel;
    Label13: TLabel;
    LUpdateTime: TLabel;
    ImageList1: TImageList;
    eErrorInfo: TEdit;
    OPCServer: TdOPCServer;
    procedure ServerComboDropDown(Sender: TObject);
    procedure bConnectClick(Sender: TObject);
    procedure bDisconnectClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Timer1Timer(Sender: TObject);
    procedure bAddClick(Sender: TObject);
    procedure bChangeClick(Sender: TObject);
    procedure bDeleteClick(Sender: TObject);
    procedure TreeView1Click(Sender: TObject);
    procedure OpcServerServerShutdown(Sender: TObject; Reason: String);
    procedure OpcServerError(Sender: TObject; Errorstring: String;
      var RaiseException: Boolean);
  private
    procedure ClearInfoLabel;
    function  GetSelectedGroup: tdOPCGroup;

  public
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.ServerComboDropDown(Sender: TObject);
begin
  if trim(ServerCombo.Items.Text) = '' then  // if no OPC Servernames loaded
  begin
    Screen.Cursor := crHourGlass;
    GetOPCServers(ServerCombo.Items);        // get Servernames from registry
    Screen.Cursor := crDefault;
  end;
end;

procedure TForm1.bConnectClick(Sender: TObject);
begin
   OpcServer.Active     := false;              // disconnect from old server
   EErrorInfo.Text      := '';
   OpcServer.ServerName := ServerCombo.Text;   // set Servername from Combobox
   OpcServer.Active     := true;               // connect to Server
   Timer1.Enabled       := true;
end;

procedure TForm1.bDisconnectClick(Sender: TObject);
begin
  OpcServer.Active := false;
end;

function TForm1.GetSelectedGroup: tdOPCGroup;
var
  Node : TTreeNode;
begin
  result := nil;
  Node := TreeView1.Selected;
  if Node <> nil then
    result := OPCServer.OPCGroups.GetOPCGroup(Node.Text); // get Group
end;


procedure TForm1.ClearInfoLabel;
var
  i : integer;
  C : TComponent;
begin
  for i := 0 to ComponentCount-1 do        // clears all Infolabels
  begin
    C := Components[i];
    if (C.Tag = 1) and (C is TLabel) then
      TLabel(C).Caption := '';
  end;
  LServer.Caption := 'not connected';
end;


procedure TForm1.FormShow(Sender: TObject);
begin
  ClearInfoLabel;
end;

procedure TForm1.Timer1Timer(Sender: TObject);
var
  Group: tdOPCGroup;
  State: TGroupStateRec;
  ServerState : TServerStateRec;
begin
  Timer1.Enabled := false;        // no timer event
  if OPCServer.Active then
  begin
    try
      Group := GetSelectedGroup;  // selected Group from Treeview
      if Group <> nil then
      begin
         Group.GetState(State); // get state direct from OPC Server
         LName.Caption     := State.Name;
         LUpdate.Caption   := IntToStr(State.UpdateRate);
         LDeadBand.Caption := format('%.2f',[State.DeadBand]);
         if State.IsActive then
           LActive.Caption := 'active'
         else
           LActive.Caption := 'not active';
      end
      else ClearInfoLabel;
      ServerState         := OpcServer.GetState;
      LServer.Caption     := OpcServer.Servername;
      LState.Caption      := ServerstatetoStr(ServerState.ServerState);
      LInfo.Caption       := ServerState.VendorInfo;
      LStartTime.Caption  := DateTimetoStr(ServerState.StartTime);
      LUpdateTime.Caption := DateTimetoStr(ServerState.LastUpdateTime);
      Timer1.Enabled := true;
    except
      Timer1.Enabled   := false;   // no Timer Event
      OPCServer.Active := false;   // if error then disconnect from OPC Server
      ClearInfoLabel;              // Clear infolabels
    end;
  end
  else
    ClearInfoLabel;
end;

// adds a new group
procedure TForm1.bAddClick(Sender: TObject);
begin
  OPCServer.OPCGroupDefault.UpdateRate :=  StrToInt(eUpdateRate.Text);
  OPCServer.OPCGroupDefault.DeadBand   :=  StrToFloat(eDeadBand.Text);
  OPCServer.OPCGroupDefault.IsActive   :=  cActive.Checked;
  OPCServer.OPCGroups.Add(eGroupName.Text);
  TreeView1.Items.AddChild(nil,eGroupName.Text);
end;


// change group properties
procedure TForm1.bChangeClick(Sender: TObject);
var
  Group : TdOPCGroup;
  Node : TTreeNode;
begin
  Node := TreeView1.Selected;
  if Node <> nil then
  begin
    Group := OPCServer.OPCGroups.GetOPCGroup(Node.Text); // get Group
    if Group <> nil then
    begin
      Group.Name       :=  EGroupName.Text;
      Node.Text        :=  EGroupName.Text;
      Group.UpdateRate :=  StrToInt(eUpdateRate.Text);
      Group.DeadBand   :=  StrToFloat(eDeadBand.Text);
      Group.IsActive   :=  cActive.Checked;
    end
    else ShowMessage('ERROR');
  end
  else ShowMessage('First select Group');
end;

// delete a group
procedure TForm1.bDeleteClick(Sender: TObject);
var
  Node : TTreeNode;
begin
  Node := TreeView1.Selected;
  if Node <> nil then
  begin
    OPCServer.OPCGroups.Remove(Node.Text);  // delete from Server
    Treeview1.Items.Delete(Node);           // delete from Tree
  end;
end;

// Show selected group properties
procedure TForm1.TreeView1Click(Sender: TObject);
var
  Group : TdOPCGroup;
begin
  Group := GetSelectedGroup;
  if Group <> nil then
  begin
    eGroupName.Text   := Group.Name;
    eUpdateRate.Text  := IntToStr(Group.UpdateRate);
    eDeadBand.Text    := format('%.2f',[Group.DeadBand]);
    cActive.Checked   := Group.IsActive;
  end
  else
    EGroupName.Text := '';
end;

// Server sends a Shutdown request
procedure TForm1.OpcServerServerShutdown(Sender: TObject; Reason: String);
begin
  Timer1.Enabled := false;   // no timer
  ClearInfoLabel;            // clear Infolables
  EErrorInfo.Text :='Disconnect from Server: ' + Reason;
end;

// Shows Errorstring in Editfield
procedure TForm1.OpcServerError(Sender: TObject; Errorstring: String;
  var RaiseException: Boolean);
begin
  EErrorInfo.Text := ErrorString;
end;

end.
